@@ -5,11 +5,21 @@ import sys
 import os
 import tempfile
 import configparser
+import re
+import gnupg
 
-logg = logging.getLogger()
+from .error import DecryptError
+
+logg = logging.getLogger('confini')
 
 current_config = None
 
+gpg = gnupg.GPG(
+    verbose=False,
+    use_agent=True,
+        )
+gpg.encoding = 'utf-8'
+
 
 def set_current(conf, description=''):
     global current_config
@@ -21,13 +31,18 @@ class Config:
 
     parser = configparser.ConfigParser(strict=True)
 
-    def __init__(self, config_dir):
+    def __init__(self, config_dir, env_prefix=None, decrypt=True):
         if not os.path.isdir(config_dir):
             raise OSError('{} is not a directory'.format(config_dir))
         self.dir = os.path.realpath(config_dir)
         self.required = {}
         self.censored = {}
         self.store = {}
+        self.decrypt = decrypt
+        self.env_prefix = None
+        if env_prefix != None:
+            logg.info('using prefix {} for environment variable override matches'.format(env_prefix))
+            self.env_prefix = '{}_'.format(env_prefix)
 
 
     def add(self, value, constant_name):
@@ -68,7 +83,15 @@ class Config:
         for s in self.parser.sections():
             for k in self.parser[s]:
                 cn = Config.to_constant_name(k, s)
-                self.add(os.environ.get(cn, self.parser[s][k]), cn)
+                cn_env = cn
+                if self.env_prefix != None:
+                    cn_env = self.env_prefix + cn
+                val = os.environ.get(cn_env)
+                if val == None or val == '':
+                    val = self.parser[s][k]
+                else:
+                    logg.info('environment variable {} overrides {}'.format(cn_env, cn))
+                self.add(val, cn)
 
 
     def process(self, set_as_current=False):
@@ -77,8 +100,10 @@ class Config:
         tmp = tempfile.NamedTemporaryFile(delete=False)
         tmpname = tmp.name
         for filename in os.listdir(self.dir):
-            if not '.ini' in filename:
+            if re.match(r'.+\.ini$', filename) == None:
+                logg.debug('skipping file {}'.format(filename))
                 continue
+            logg.info('reading file {}'.format(filename))
             f = open(os.path.join(self.dir, filename), 'rb')
             while 1:
                 data = f.read()
@@ -94,8 +119,39 @@ class Config:
             set_current(self, description=self.dir)
 
 
-    def get(self, k):
-        return self.store.get(k)
+
+    def _decrypt(self, k, v):
+        if self.decrypt:
+            m = re.match(r'^\!gpg\((.*)\)', v)
+            if m != None:
+                filename = m.group(1)
+                if filename[0] != '/':
+                    filename = os.path.join(self.dir, filename)
+                f = open(filename, 'rb')
+                logg.debug('decrypting entry {} in file {}'.format(k, f))
+                d = gpg.decrypt_file(f)
+                if not d.ok:
+                    raise DecryptError()
+                v = str(d)
+                f.close()
+        return v
+
+
+    def get(self, k, default=None):
+        v = self.store.get(k)
+        if v == None:
+            if default != None:
+                logg.debug('returning default value for {}'.format(k))
+            return default
+        return self._decrypt(k, v)
+
+
+    def true(self, k):
+        v = self.store.get(k)
+        d = self._decrypt(k, v)
+        if d.lower() not in ['true', 'false', '0', '1', 'on', 'off']:
+            raise ValueError('{} not a boolean value'.format(k))
+        return d.lower() in ['true', '1', 'on']
 
 
     def __str__(self):
@@ -126,6 +182,4 @@ def config_from_environment():
 
 
 def config_dir_from_environment():
-    return os.environ.get('CIC_CONFIG_DIR')
-
-
+    return os.environ.get('CONFINI_DIR')
